Struct CompressedEdwardsY

pub struct CompressedEdwardsY(pub [u8; 32]);
In “Edwards y” / “Ed25519” format, the curve point \((x,y)\) is determined by the \(y\)-coordinate and the sign of \(x\).

The first 255 bits of a CompressedEdwardsY represent the \(y\)-coordinate. The high bit of the 32nd byte gives the sign of \(x\).

Tuple Fields

0: [u8; 32]

Implementations

Source

impl CompressedEdwardsY

pub const fn as_bytes(&self) -> &[u8; 32]

View this CompressedEdwardsY as an array of bytes.

✓✓ Lean specification
theorem as_bytes_spec (self : CompressedEdwardsY) :
    as_bytes self ⦃ (result : Array U8 32#usize) =>
      result = self ⦄
pub const fn to_bytes(&self) -> [u8; 32]

Copy this CompressedEdwardsY to an array of bytes.

pub fn decompress(&self) -> Option<EdwardsPoint>

Attempt to decompress to an EdwardsPoint.

Returns None if the input is not the \(y\)-coordinate of a curve point.

✓✓ Lean specification
theorem decompress_spec (self : CompressedEdwardsY) :
    decompress self ⦃ (result : Option EdwardsPoint) =>
      let y : CurveField := ((U8x32_as_Nat self % 2 ^ 255 : Nat) : CurveField)
      let x_sign_bit := self[31]!.val.testBit 7
      (result.isSome ↔ ∃ pt : Point Ed25519, pt.y = y) ∧
      (∀ ep, result = some ep →
        ep.IsValid ∧
        ep.Y.toField = y ∧
        Field51_as_Nat ep.Y ≡ (U8x32_as_Nat self % 2 ^ 255) [MOD p] ∧
        ep.Z.toField = 1 ∧
        Field51_as_Nat ep.Z % p = 1 ∧
        (y ^ 21 →
          (x_sign_bit ↔ (Field51_as_Nat ep.X % p) % 2 = 1))) ⦄
Source

impl CompressedEdwardsY

pub fn from_slice(bytes: &[u8]) -> Result<CompressedEdwardsY, TryFromSliceError>

Construct a CompressedEdwardsY from a slice of bytes.

Errors

Returns TryFromSliceError if the input bytes slice does not have a length of 32.

✓✓ Lean specification
theorem from_slice_spec
    (bytes : Slice U8) :
    from_slice bytes ⦃
      (result : core.result.Result CompressedEdwardsY core.array.TryFromSliceError) =>
      (bytes.length = 32 → ∃ cey : CompressedEdwardsY, result = .Ok cey ∧ cey.val = bytes.val) ∧
      (bytes.length ≠ 32 → result = .Err ()) ⦄