pub struct MontgomeryPoint(pub [u8; 32]);Expand description
Holds the \(u\)-coordinate of a point on the Montgomery form of Curve25519 or its twist.
Tuple Fields
0: [u8; 32]Implementations
Sourceimpl MontgomeryPoint
impl MontgomeryPoint
Sourcepub fn mul_base(scalar: &Scalar) -> Self
pub fn mul_base(scalar: &Scalar) -> Self
Fixed-base scalar multiplication (i.e. multiplication by the base point).
theorem mul_base_spec (scalar : scalar.Scalar)
(h_s_canonical : U8x32_as_Nat scalar.bytes < 2 ^ 255) :
mul_base scalar ⦃ (result : MontgomeryPoint) =>
let ep := (U8x32_as_Nat scalar.bytes) • _root_.Edwards.basepoint
if ep.y = 1 then
MontgomeryPoint.mkPoint result = T_point
else
MontgomeryPoint.mkPoint result = abs_montgomery
((U8x32_as_Nat scalar.bytes) • (fromEdwards _root_.Edwards.basepoint)) ⦄
Sourcepub fn mul_clamped(self, bytes: [u8; 32]) -> Self
pub fn mul_clamped(self, bytes: [u8; 32]) -> Self
Multiply this point by clamp_integer(bytes). For a description of clamping, see
clamp_integer.
theorem mul_clamped_spec (P : MontgomeryPoint) (bytes : Array U8 32#usize)
(hP_bound : U8x32_as_Nat P < 2 ^ 255) :
mul_clamped P bytes ⦃ (result : MontgomeryPoint) =>
(∃ clamped_scalar,
h ∣ U8x32_as_Nat clamped_scalar ∧
U8x32_as_Nat clamped_scalar < 2 ^ 255 ∧
2 ^ 254 ≤ U8x32_as_Nat clamped_scalar ∧
let m := (U8x32_as_Nat clamped_scalar)
MontgomeryPoint.mkPoint result = m • (MontgomeryPoint.mkPoint P)) ⦄
Sourcepub fn mul_base_clamped(bytes: [u8; 32]) -> Self
pub fn mul_base_clamped(bytes: [u8; 32]) -> Self
Multiply the basepoint by clamp_integer(bytes). For a description of clamping, see
clamp_integer.
theorem mul_base_clamped_spec (bytes : Array U8 32#usize) :
mul_base_clamped bytes ⦃ (result : MontgomeryPoint) =>
(∃ clamped_scalar_nat,
h ∣ clamped_scalar_nat ∧
clamped_scalar_nat < 2 ^ 255 ∧
2 ^ 254 ≤ clamped_scalar_nat ∧
(if (clamped_scalar_nat • Edwards.basepoint).y = 1 then
MontgomeryPoint.mkPoint result = T_point
else MontgomeryPoint.mkPoint result =
abs_montgomery (clamped_scalar_nat • fromEdwards _root_.Edwards.basepoint))) ⦄
Sourcepub fn mul_bits_be(&self, bits: impl Iterator<Item = bool>) -> MontgomeryPoint
Available on non-verify only.
pub fn mul_bits_be(&self, bits: impl Iterator<Item = bool>) -> MontgomeryPoint
verify only.Given self \( = u_0(P) \), and a big-endian bit representation of an integer
\(n\), return \( u_0([n]P) \). This is constant time in the length of bits.
NOTE: You probably do not want to use this function. Almost every protocol built on
Curve25519 uses clamped multiplication, explained
here.
When in doubt, use Self::mul_clamped.
Sourcepub const fn as_bytes(&self) -> &[u8; 32]
pub const fn as_bytes(&self) -> &[u8; 32]
View this MontgomeryPoint as an array of bytes.
theorem as_bytes_spec (self : MontgomeryPoint) :
as_bytes self ⦃ (result : MontgomeryPoint) =>
result = self ⦄
Sourcepub const fn to_bytes(&self) -> [u8; 32]
pub const fn to_bytes(&self) -> [u8; 32]
Convert this MontgomeryPoint to an array of bytes.
theorem to_bytes_spec (self : MontgomeryPoint) :
to_bytes self ⦃ (result : MontgomeryPoint) =>
result = self ⦄
Sourcepub fn to_edwards(&self, sign: u8) -> Option<EdwardsPoint>
pub fn to_edwards(&self, sign: u8) -> Option<EdwardsPoint>
Attempt to convert to an EdwardsPoint, using the supplied
choice of sign for the EdwardsPoint.
Inputs
sign: au8donating the desired sign of the resultingEdwardsPoint.0denotes positive and1negative.
Return
-
Some(EdwardsPoint)ifselfis the \(u\)-coordinate of a point on (the Montgomery form of) Curve25519; -
Noneifselfis the \(u\)-coordinate of a point on the twist of (the Montgomery form of) Curve25519;
theorem to_edwards_spec (mp : MontgomeryPoint) (sign : U8) :
to_edwards mp sign ⦃ (result : Option edwards.EdwardsPoint) =>
(∀ ep, result = some ep →
∃ Z_inv,
field.FieldElement51.invert ep.Z = ok Z_inv ∧
let u := U8x32_as_Nat mp % 2^255
let y := Field51_as_Nat ep.Y * Field51_as_Nat Z_inv % p
(y * ((u + 1) % p)) % p = ((u + p - 1) % p) % p ∧
ep.IsValid ) ⦄