Struct EdwardsPoint

pub struct EdwardsPoint { /* private fields */ }
An EdwardsPoint represents a point on the Edwards form of Curve25519.

Implementations

Source

impl EdwardsPoint

pub fn to_montgomery(&self) -> MontgomeryPoint

Convert this EdwardsPoint on the Edwards model to the corresponding MontgomeryPoint on the Montgomery model.

This function has one exceptional case; the identity point of the Edwards curve is sent to the 2-torsion point \((0,0)\) on the Montgomery curve.

Note that this is a one-way conversion, since the Montgomery model does not retain sign information.

✓✓ Lean specification
theorem to_montgomery_spec (self : EdwardsPoint)
    (hvalid : self.IsValid)
    (h_Y_bounds : ∀ i < 5, self.Y[i]!.val < 2 ^ 53)
    (h_Z_bounds : ∀ i < 5, self.Z[i]!.val < 2 ^ 53) :
    to_montgomery self ⦃ (mp : montgomery.MontgomeryPoint) =>
      let Y := Field51_as_Nat self.Y
      let Z := Field51_as_Nat self.Z
      let u := U8x32_as_Nat mp
      if Z % p = Y % p then
        u % p = 0 ∧ (∀ n : ℕ, fromEdwards (n • self.toPoint) = 0)
      else
        ((u * Z) % p = (u * Y + (Z + Y)) % p) ∧
        abs_montgomery (fromEdwards self.toPoint) = MontgomeryPoint.mkPoint mp ⦄
pub fn to_montgomery_batch(eds: &[Self]) -> Vec<MontgomeryPoint>

Available on crate feature alloc and non-verify only.

Converts a large batch of points to Edwards at once. This has the same behavior on identity elements as Self::to_montgomery.

pub fn compress(&self) -> CompressedEdwardsY

Compress this point to CompressedEdwardsY format.

✓✓ Lean specification
theorem compress_spec (self : EdwardsPoint) (hself : self.IsValid) :
    compress self ⦃ (result : CompressedEdwardsY) =>
      U8x32_as_Nat result = compress_edwards_pure self.toPoint ⦄
pub fn compress_batch(inputs: &[EdwardsPoint]) -> Vec<CompressedEdwardsY>

Available on crate feature alloc and non-verify only.

Compress several EdwardsPoints into CompressedEdwardsY format, using a batch inversion for a significant speedup.

Source

impl EdwardsPoint

pub fn mul_base(scalar: &Scalar) -> Self

Fixed-base scalar multiplication by the Ed25519 base point.

Uses precomputed basepoint tables when the precomputed-tables feature is enabled, trading off increased code size for ~4x better performance.

✓✓ Lean specification
theorem mul_base_spec (scalar : scalar.Scalar)
    (h_s_canonical : U8x32_as_Nat scalar.bytes < 2 ^ 255) :
    mul_base scalar ⦃ (res : EdwardsPoint) =>
      EdwardsPoint.IsValid res ∧
      res.toPoint = (U8x32_as_Nat scalar.bytes) • _root_.Edwards.basepoint ⦄
pub fn mul_clamped(self, bytes: [u8; 32]) -> Self

Multiply this point by clamp_integer(bytes). For a description of clamping, see clamp_integer.

✓✓ Lean specification
theorem mul_clamped_spec (self : EdwardsPoint) (bytes : Array U8 32#usize)
    (h_self_valid : self.IsValid) :
    mul_clamped self bytes ⦃ (result : EdwardsPoint) =>
      result.IsValid ∧
      (∃ clamped_scalar,
        h ∣ U8x32_as_Nat clamped_scalar ∧
        U8x32_as_Nat clamped_scalar < 2 ^ 2552 ^ 254 ≤ U8x32_as_Nat clamped_scalar ∧
        result.toPoint = (U8x32_as_Nat clamped_scalar) • self.toPoint) ⦄
pub fn mul_base_clamped(bytes: [u8; 32]) -> Self

Multiply the basepoint by clamp_integer(bytes). For a description of clamping, see clamp_integer.

✓✓ Lean specification
theorem mul_base_clamped_spec (bytes : Array U8 32#usize) :
    mul_base_clamped bytes ⦃ (result : EdwardsPoint) =>
      EdwardsPoint.IsValid result ∧
      (∃ clamped_scalar,
        h ∣ U8x32_as_Nat clamped_scalar ∧
        U8x32_as_Nat clamped_scalar < 2 ^ 2552 ^ 254 ≤ U8x32_as_Nat clamped_scalar ∧
        result.toPoint = ((U8x32_as_Nat clamped_scalar) • _root_.Edwards.basepoint)) ⦄
Source

impl EdwardsPoint

pub fn vartime_double_scalar_mul_basepoint( a: &Scalar, A: &EdwardsPoint, b: &Scalar, ) -> EdwardsPoint

Available on non-verify only.

Compute \(aA + bB\) in variable time, where \(B\) is the Ed25519 basepoint.

Source

impl EdwardsPoint

pub fn mul_by_cofactor(&self) -> EdwardsPoint

Multiply by the cofactor: return \([8]P\).

✓✓ Lean specification
theorem mul_by_cofactor_spec (self : EdwardsPoint) (hself : self.IsValid) :
    mul_by_cofactor self ⦃ (result : EdwardsPoint) =>
      result.IsValid ∧
      result.toPoint = h • self.toPoint ⦄
pub fn is_small_order(&self) -> bool

Determine if this point is of small order.

Return
  • true if self is in the torsion subgroup \( \mathcal E[8] \);
  • false if self is not in the torsion subgroup \( \mathcal E[8] \).
Example
use curve25519_dalek::constants;

// Generator of the prime-order subgroup
let P = constants::ED25519_BASEPOINT_POINT;
// Generator of the torsion subgroup
let Q = constants::EIGHT_TORSION[1];

// P has large order
assert_eq!(P.is_small_order(), false);

// Q has small order
assert_eq!(Q.is_small_order(), true);
✓✓ Lean specification
theorem is_small_order_spec (self : EdwardsPoint) (hself : self.IsValid) :
    is_small_order self ⦃ (result : Bool) =>
      (result ↔ h • self.toPoint = 0) ⦄
pub fn is_torsion_free(&self) -> bool

Determine if this point is “torsion-free”, i.e., is contained in the prime-order subgroup.

Return
  • true if self has zero torsion component and is in the prime-order subgroup;
  • false if self has a nonzero torsion component and is not in the prime-order subgroup.
Example
use curve25519_dalek::constants;

// Generator of the prime-order subgroup
let P = constants::ED25519_BASEPOINT_POINT;
// Generator of the torsion subgroup
let Q = constants::EIGHT_TORSION[1];

// P is torsion-free
assert_eq!(P.is_torsion_free(), true);

// P + Q is not torsion-free
assert_eq!((P+Q).is_torsion_free(), false);
✓✓ Lean specification
theorem is_torsion_free_spec
    (self : EdwardsPoint) (hself : self.IsValid) :
    is_torsion_free self ⦃ (result : Bool) =>
      (result ↔ L • self.toPoint = 0) ⦄