pub struct RistrettoPoint(/* private fields */);Expand description
A RistrettoPoint represents a point in the Ristretto group for
Curve25519. Ristretto, a variant of Decaf, constructs a
prime-order group as a quotient group of a subgroup of (the
Edwards form of) Curve25519.
Internally, a RistrettoPoint is implemented as a wrapper type
around EdwardsPoint, with custom equality, compression, and
decompression routines to account for the quotient. This means that
operations on RistrettoPoints are exactly as fast as operations on
EdwardsPoints.
Implementations
Sourceimpl RistrettoPoint
impl RistrettoPoint
Sourcepub fn compress(&self) -> CompressedRistretto
pub fn compress(&self) -> CompressedRistretto
Compress this point using the Ristretto encoding.
theorem compress_spec (self : RistrettoPoint) (h : self.IsValid) :
compress self ⦃ (result : CompressedRistretto) =>
result.IsValid ∧
math.compress_pure self.toPoint = U8x32_as_Nat result ⦄
Sourcepub fn double_and_compress_batch<'a, I>(points: I) -> Vec<CompressedRistretto>where
I: IntoIterator<Item = &'a RistrettoPoint>,
Available on crate feature alloc and non-verify only.
pub fn double_and_compress_batch<'a, I>(points: I) -> Vec<CompressedRistretto>where
I: IntoIterator<Item = &'a RistrettoPoint>,
alloc and non-verify only.Double-and-compress a batch of points. The Ristretto encoding is not batchable, since it requires an inverse square root.
However, given input points \( P_1, \ldots, P_n, \) it is possible to compute the encodings of their doubles \( \mathrm{enc}( [2]P_1), \ldots, \mathrm{enc}( [2]P_n ) \) in a batch.
use rand_core::OsRng;
let mut rng = OsRng;
let points: Vec<RistrettoPoint> =
(0..32).map(|_| RistrettoPoint::random(&mut rng)).collect();
let compressed = RistrettoPoint::double_and_compress_batch(&points);
for (P, P2_compressed) in points.iter().zip(compressed.iter()) {
assert_eq!(*P2_compressed, (P + P).compress());
}Sourcepub fn from_uniform_bytes(bytes: &[u8; 64]) -> RistrettoPoint
pub fn from_uniform_bytes(bytes: &[u8; 64]) -> RistrettoPoint
Construct a RistrettoPoint from 64 bytes of data.
If the input bytes are uniformly distributed, the resulting point will be uniformly distributed over the group, and its discrete log with respect to other points should be unknown.
Implementation
This function splits the input array into two 32-byte halves, takes the low 255 bits of each half mod p, applies the Ristretto-flavored Elligator map to each, and adds the results.
theorem from_uniform_bytes_spec (bytes : Array U8 64#usize) :
from_uniform_bytes bytes ⦃ (result : RistrettoPoint) =>
result.IsValid ∧
result.toPoint =
(elligator_ristretto_flavor_pure (field_from_bytes (bytes_lower bytes))).val +
(elligator_ristretto_flavor_pure (field_from_bytes (bytes_upper bytes))).val ⦄
Sourceimpl RistrettoPoint
impl RistrettoPoint
Sourcepub fn mul_base(scalar: &Scalar) -> Self
pub fn mul_base(scalar: &Scalar) -> Self
Fixed-base scalar multiplication by the Ristretto base point.
Uses precomputed basepoint tables when the precomputed-tables feature
is enabled, trading off increased code size for ~4x better performance.
theorem mul_base_spec (s : scalar.Scalar) (h_s_canonical : U8x32_as_Nat s.bytes < 2 ^ 255) :
mul_base s ⦃ (result : RistrettoPoint) =>
result.IsValid ∧
result.toPoint = (U8x32_as_Nat s.bytes) • _root_.Edwards.basepoint ⦄
Sourceimpl RistrettoPoint
impl RistrettoPoint
Sourcepub fn vartime_double_scalar_mul_basepoint(
a: &Scalar,
A: &RistrettoPoint,
b: &Scalar,
) -> RistrettoPoint
Available on non-verify only.
pub fn vartime_double_scalar_mul_basepoint( a: &Scalar, A: &RistrettoPoint, b: &Scalar, ) -> RistrettoPoint
verify only.Compute \(aA + bB\) in variable time, where \(B\) is the Ristretto basepoint.