pub struct Scalar { /* private fields */ }Expand description
The Scalar struct holds an element of \(\mathbb Z / \ell\mathbb Z \).
Implementations
Sourceimpl Scalar
impl Scalar
Sourcepub fn from_bytes_mod_order(bytes: [u8; 32]) -> Scalar
pub fn from_bytes_mod_order(bytes: [u8; 32]) -> Scalar
Construct a Scalar by reducing a 256-bit little-endian integer
modulo the group order \( \ell \).
theorem from_bytes_mod_order_spec (bytes : Array U8 32#usize) :
from_bytes_mod_order bytes ⦃ (s : Scalar) =>
U8x32_as_Nat s.bytes ≡ U8x32_as_Nat bytes [MOD L] ∧
U8x32_as_Nat s.bytes < L ⦄
Sourcepub fn from_bytes_mod_order_wide(input: &[u8; 64]) -> Scalar
pub fn from_bytes_mod_order_wide(input: &[u8; 64]) -> Scalar
Construct a Scalar by reducing a 512-bit little-endian integer
modulo the group order \( \ell \).
theorem from_bytes_mod_order_wide_spec (input : Array U8 64#usize) :
from_bytes_mod_order_wide input ⦃ (result : Scalar) =>
U8x32_as_Nat result.bytes ≡ U8x64_as_Nat input [MOD L] ∧
U8x32_as_Nat result.bytes < L ⦄
Sourcepub fn from_canonical_bytes(bytes: [u8; 32]) -> CtOption<Scalar>
pub fn from_canonical_bytes(bytes: [u8; 32]) -> CtOption<Scalar>
Attempt to construct a Scalar from a canonical byte representation.
Return
Some(s), wheresis theScalarcorresponding tobytes, ifbytesis a canonical byte representation modulo the group order \( \ell \);Noneifbytesis not a canonical byte representation.
theorem from_canonical_bytes_spec (bytes : Array U8 32#usize) :
from_canonical_bytes bytes ⦃ (s : subtle.CtOption Scalar) =>
(U8x32_as_Nat bytes < L → s.is_some = Choice.one ∧ s.value.bytes = bytes) ∧
(L ≤ U8x32_as_Nat bytes → s.is_some = Choice.zero) ⦄
Sourceimpl Scalar
impl Scalar
Sourcepub const fn to_bytes(&self) -> [u8; 32]
pub const fn to_bytes(&self) -> [u8; 32]
Convert this Scalar to its underlying sequence of bytes.
Example
use curve25519_dalek::scalar::Scalar;
let s: Scalar = Scalar::ZERO;
assert!(s.to_bytes() == [0u8; 32]);theorem to_bytes_spec (s : Scalar) :
to_bytes s ⦃ (a : Array U8 32#usize) =>
a = s.bytes ∧
mk a = s ⦄
Sourcepub const fn as_bytes(&self) -> &[u8; 32]
pub const fn as_bytes(&self) -> &[u8; 32]
View the little-endian byte encoding of the integer representing this Scalar.
Example
use curve25519_dalek::scalar::Scalar;
let s: Scalar = Scalar::ZERO;
assert!(s.as_bytes() == &[0u8; 32]);theorem as_bytes_spec (s : Scalar) :
as_bytes s ⦃ (result : Array U8 32#usize) =>
result = s.bytes ∧
mk result = s ⦄
Sourcepub fn invert(&self) -> Scalar
pub fn invert(&self) -> Scalar
Given a nonzero Scalar, compute its multiplicative inverse.
Warning
self MUST be nonzero. If you cannot
prove that this is the case, you SHOULD NOT USE THIS
FUNCTION.
Returns
The multiplicative inverse of the this Scalar.
Example
use curve25519_dalek::scalar::Scalar;
// x = 2238329342913194256032495932344128051776374960164957527413114840482143558222
let X: Scalar = Scalar::from_bytes_mod_order([
0x4e, 0x5a, 0xb4, 0x34, 0x5d, 0x47, 0x08, 0x84,
0x59, 0x13, 0xb4, 0x64, 0x1b, 0xc2, 0x7d, 0x52,
0x52, 0xa5, 0x85, 0x10, 0x1b, 0xcc, 0x42, 0x44,
0xd4, 0x49, 0xf4, 0xa8, 0x79, 0xd9, 0xf2, 0x04,
]);
// 1/x = 6859937278830797291664592131120606308688036382723378951768035303146619657244
let XINV: Scalar = Scalar::from_bytes_mod_order([
0x1c, 0xdc, 0x17, 0xfc, 0xe0, 0xe9, 0xa5, 0xbb,
0xd9, 0x24, 0x7e, 0x56, 0xbb, 0x01, 0x63, 0x47,
0xbb, 0xba, 0x31, 0xed, 0xd5, 0xa9, 0xbb, 0x96,
0xd5, 0x0b, 0xcd, 0x7a, 0x3f, 0x96, 0x2a, 0x0f,
]);
let inv_X: Scalar = X.invert();
assert!(XINV == inv_X);
let should_be_one: Scalar = &inv_X * &X;
assert!(should_be_one == Scalar::ONE);theorem invert_spec (self : Scalar) (h : U8x32_as_Nat self.bytes % L ≠ 0) :
invert self ⦃ (result : Scalar) =>
U8x32_as_Nat self.bytes * U8x32_as_Nat result.bytes ≡ 1 [MOD L] ⦄
Sourcepub fn batch_invert(inputs: &mut [Scalar]) -> Scalar
Available on crate feature alloc only.
pub fn batch_invert(inputs: &mut [Scalar]) -> Scalar
alloc only.Given a slice of nonzero (possibly secret) Scalars,
compute their inverses in a batch.
Return
Each element of inputs is replaced by its inverse.
The product of all inverses is returned.
Warning
All input Scalars MUST be nonzero. If you cannot
prove that this is the case, you SHOULD NOT USE THIS
FUNCTION.
Example
let mut scalars = [
Scalar::from(3u64),
Scalar::from(5u64),
Scalar::from(7u64),
Scalar::from(11u64),
];
let allinv = Scalar::batch_invert(&mut scalars);
assert_eq!(allinv, Scalar::from(3*5*7*11u64).invert());
assert_eq!(scalars[0], Scalar::from(3u64).invert());
assert_eq!(scalars[1], Scalar::from(5u64).invert());
assert_eq!(scalars[2], Scalar::from(7u64).invert());
assert_eq!(scalars[3], Scalar::from(11u64).invert());theorem batch_invert_spec
(inputs : Slice scalar.Scalar)
(vals : ℕ → ℕ)
(h_vals_lt : ∀ j < inputs.val.length, vals j < L)
(h_vals_nz : ∀ j < inputs.val.length, vals j % L ≠ 0)
(h_vals_def : ∀ j < inputs.val.length,
SliceScalarAt inputs j (fun b => U8x32_as_Nat b = vals j)) :
batch_invert inputs ⦃ (result : scalar.Scalar × Slice scalar.Scalar) =>
U8x32_as_Nat result.1.bytes * PrefixProd vals inputs.val.length ≡ 1 [MOD L] ∧
∀ j < inputs.val.length,
SliceScalarAt result.2 j
(fun b => U8x32_as_Nat b * vals j ≡ 1 [MOD L]) ⦄